#include "CivGraphList.h"
#include "CivPgConn.h"
#include "CivDbConn.h"

template <class VertexType, class EdgeType>
ALGraph<VertexType, EdgeType>::ALGraph()
{

}

template <class VertexType, class EdgeType>
void ALGraph<VertexType, EdgeType>::createGraph(CivDbConn* dbConn)
{
    if (dbConn == nullptr)
        return;

    
}

template <class VertexType, class EdgeType>
ALGraph<VertexType, EdgeType>::~ALGraph()
{
    for (auto iter = m_vertexArray.begin(); iter != m_vertexArray.end(); iter++)
    {
        Edge<weight>* p = iter->pAdjEdges;
        while (NULL != p)
        {
            iter->pAdjEdges = p->pNextEdge;
            delete p;
            p = iter->pAdjEdges;
        }
    }
    if (!m_vertexArray.empty())
    {
        m_vertexArray.clear();
    }
}

template <class VertexType, class EdgeType>
bool ALGraph<VertexType, EdgeType>::insertAVertex(IN const VertexType& vertexName)
{
    Vertex<VertexType, EdgeType> VertexInstance(vertexName, NULL);
    m_vertexArray.push_back(VertexInstance);

    return true;
}

template <class VertexType, class EdgeType>
bool ALGraph<VertexType, EdgeType>::insertAEdge(IN const VertexType vertexName1,
    IN const VertexType vertexName2, IN const EdgeType edgeWeight)
{
    int v1 = getVertexIndex(vertexName1);
    if (-1 == v1)
    {
        cerr << "There is no vertex 1" << endl;
        return false;
    }

    int v2 = getVertexIndex(vertexName2);
    if (-1 == v2)
    {
        cerr << "There is no vertex 2" << endl;
        return false;
    }

    Edge<weight>* p = m_vertexArray.at(v1).pAdjEdges;
    while (p != NULL && p->nDestVertex != v2)
    {
        p = p->pNextEdge;
    }

    if (NULL == p)
    {
        p = new Edge<weight>(v2, edgeWeight, m_vertexArray.at(v1).pAdjEdges);
        m_vertexArray.at(v1).pAdjEdges = p;
        return true;
    }

    if (v2 == p->nDestVertex)
    {
        Edge<weight>* q = p;
        p = new Edge<weight>(v2, edgeWeight, q->pNextEdge);
        q->pNextEdge = p;
        return true;
    }

    return false;
}

template <class VertexType, class EdgeType>
bool ALGraph<VertexType, EdgeType>::removeAEdge(IN const VertexType& vertexName1, 
    IN const VertexType& vertexName2, IN const EdgeType& edgeWeight)
{
    int v1 = getVertexIndex(vertexName1);
    if (-1 == v1)
    {
        cerr << "There is no vertex 1" << endl;
        return false;
    }

    int v2 = getVertexIndex(vertexName2);
    if (-1 == v2)
    {
        cerr << "There is no vertex 2" << endl;
        return false;
    }

    Edge<EdgeType>* p = m_vertexArray.at(v1).pAdjEdges;
    Edge<EdgeType>* q = NULL;
    while (p != NULL && p->nDestVertex != v2)
    {
        q = p;
        p = p->pNextEdge;
    }

    if (NULL == p)
    {
        cerr << "Edge is not found" << endl;
        return false;
    }

    while (edgeWeight != p->edgeWeight && p->nDestVertex == v2)
    {
        q = p;
        p = p->pNextEdge;
    }

    if (v2 != p->nDestVertex)
    {
        cerr << "Edge is not found" << endl;
        return false;
    }

    q->pNextEdge = p->pNextEdge;
    delete p;

    return true;
}

template <class VertexType, class EdgeType>
EdgeType ALGraph<VertexType, EdgeType>::getEdgeWeight(IN const Edge<EdgeType>* pEdge)
{
    return pEdge->edgeWeight;
}

template <class VertexType, class EdgeType>
void ALGraph<VertexType, EdgeType>::getVertexEdgeWeight(IN const int v1, OUT vector<EdgeType>& DistanceArray)
{
    Edge<weight>* p = m_vertexArray.at(v1).pAdjEdges;
    int prevIndex = -1;
    weight tmp;

    while (NULL != p)
    {
        //consider the same edges exist
        if (prevIndex == p->nDestVertex)
        {
            if (tmp > p->edgeWeight)
            {
                DistanceArray[prevIndex] = p->edgeWeight;
            }
        }
        else
        {
            DistanceArray[p->nDestVertex] = p->edgeWeight;
            prevIndex = p->nDestVertex;
            tmp = p->edgeWeight;
        }

        p = p->pNextEdge;
    }
}

template <class VertexType, class EdgeType>
EdgeType ALGraph<VertexType, EdgeType>::getMinWeight(IN const VertexType vertexName1,
    IN const VertexType vertexName2)
{
    Edge<EdgeType>* pEdge = NULL;
    int v1 = getVertexIndex(vertexName1);
    if (-1 == v1)
    {
        cerr << "There is no vertex 1" << endl;
        return false;
    }

    int v2 = getVertexIndex(vertexName2);
    if (-1 == v2)
    {
        cerr << "There is no vertex 2" << endl;
        return false;
    }

    Edge<EdgeType>* p = m_vertexArray.at(v1).pAdjEdges;
    while (p != NULL && p->nDestVertex != v2)
    {
        p = p->pNextEdge;
    }
    if (NULL == p)
    {
        pEdge = NULL;
        return weight(0);
    }
    EdgeType tmp = getEdgeWeight(p);
    pEdge = p;
    while (NULL != p && v2 == p->nDestVertex)
    {
        if (tmp > getEdgeWeight(p))
        {
            tmp = getEdgeWeight(p);
            pEdge = p;
        }
        p = p->pNextEdge;
    }
    return tmp;
}

template <class   VertexType, class EdgeType>
int ALGraph<VertexType, EdgeType>::getVertexIndex(IN const VertexType vertexName)
{
    for (int i = 0; i < m_vertexArray.size(); i++)
    {
        if (vertexName == getData(i))
        {
            return i;
        }
    }
    return -1;
}

template <class   VertexType, class EdgeType>
int ALGraph<VertexType, EdgeType>::getVertexNumber()
{
    return m_vertexArray.size();
}

template <class   VertexType, class EdgeType>
VertexType ALGraph<VertexType, EdgeType>::getData(IN int index)
{
    return m_vertexArray.at(index).vertexName;
}

template <class   VertexType, class EdgeType>
int ALGraph<VertexType, EdgeType>::Dijkstra(IN const VertexType vertexName1)
{
    int sourceIndex = getVertexIndex(vertexName1);
    if (-1 == sourceIndex)
    {
        cerr << "There is no vertex " << endl;
        return false;
    }
    int nVertexNo = getVertexNumber();

    //the array to record the points have been included, if included the value is true
    //else is false
    vector<bool> vecIncludeArray;
    vecIncludeArray.assign(nVertexNo, false);
    vecIncludeArray[sourceIndex] = true;

    //the array to record the distance from vertex1
    vector<EdgeType> vecDistanceArray;
    vecDistanceArray.assign(nVertexNo, weight(INT_MAX));
    vecDistanceArray[sourceIndex] = weight(0);

    //prev array to record the previous vertex
    vector<int> vecPrevVertex;
    vecPrevVertex.assign(nVertexNo, sourceIndex);

    getVertexEdgeWeight(sourceIndex, vecDistanceArray);

    int vFrom, vTo;

    while (1)
    {
        EdgeType minWeight = weight(INT_MAX);
        vFrom = sourceIndex;
        vTo = -1;
        for (int i = 0; i < nVertexNo; i++)
        {
            if (!vecIncludeArray[i] && minWeight > vecDistanceArray[i])
            {
                minWeight = vecDistanceArray[i];
                vFrom = i;
            }
        }
        if (EdgeType(INT_MAX) == minWeight)
        {
            break;
        }
        vecIncludeArray[vFrom] = true;

        Edge<EdgeType>* p = m_vertexArray[vFrom].pAdjEdges;
        while (NULL != p)
        {
            EdgeType wFT = p->edgeWeight;
            vTo = p->nDestVertex;
            if (!vecIncludeArray[vTo] && vecDistanceArray[vTo] > wFT + vecDistanceArray[vFrom])
            {
                vecDistanceArray[vTo] = wFT + vecDistanceArray[vFrom];
                vecPrevVertex[vTo] = vFrom;
            }
            p = p->pNextEdge;
        }

    }

    //print the shortest route of all vertexes
    for (int i = 0; i < nVertexNo; i++)
    {
        if (EdgeType(INT_MAX) != vecDistanceArray[i])
        {
            cout << getData(sourceIndex) << "->" << getData(i) << ": ";
            DijkstraPrint(i, sourceIndex, vecPrevVertex);
            cout << "  " << vecDistanceArray[i];
            cout << endl;
        }
    }

    return 0;
}

template <class   VertexType, class EdgeType>
void ALGraph<VertexType, EdgeType>::DijkstraPrint(IN int index, IN int sourceIndex, IN vector<int> vecPreVertex)
{
    if (sourceIndex != index)
    {
        DijkstraPrint(vecPreVertex[index], sourceIndex, vecPreVertex);
    }
    cout << getData(index) << " ";
}

template <class   VertexType, class EdgeType >
ostream& operator<<(OUT ostream& out, IN  ALGraph<VertexType, EdgeType>& graphInstance)
{
    int vertexNo = graphInstance.getVertexNumber();
    out << "This graph has " << vertexNo << "vertexes" << endl;

    for (int i = 0; i < vertexNo; i++)
    {
        VertexType x1 = graphInstance.getData(i);
        out << x1 << ":    ";

        Edge<EdgeType>* p = graphInstance.m_vertexArray.at(i).pAdjEdges;
        while (NULL != p)
        {
            out << "(" << x1 << "," << graphInstance.getData(p->nDestVertex) << "," << p->edgeWeight << ")  ";
            p = p->pNextEdge;
        }
        out << endl;
    }
    return out;
}
