#include "CivHydrCalc.h"
#include "CivDbConnection.h"
#include "CivTypes.h"
#include "Epanet2.h"
#include "../pandaDbManager/CivCommonUtils.h"

#include <time.h> 
#include "CivBuilder.h"
#include "CivInpDirector.h"
#include "CivDbConn.h"
#include "CivPgConn.h"
#include <iostream>

CivHydrCalc::CivHydrCalc(char* uri, std::string netName)
    : mNetname(netName)
{
    mDbConn = new CivDbConnection(uri);
    mPgConn = new CivPgConn(uri);
}

CivHydrCalc::~CivHydrCalc()
{
    delete mDbConn;
}

bool CivHydrCalc::hydrSimulation(char* fileName, char* rptFile, char* binOutFile)
{
    if (!exportInp(fileName))
    {
        CivSysLog::getInstance()->error("дinpļʧ", "CivHydrCalc", __FUNCTION__);
        return false;
    }
    CivSysLog::getInstance()->info("дinpɹ", "CivHydrCalc", __FUNCTION__);

    short hydCode = 1;
    long  t(0), tstep(0);
    int   iTime(0);

    int  errcode = 0;
    errcode = ENopen(fileName, rptFile, binOutFile);
    if (errcode > 0)
    {
        ENclose();
        CivSysLog::getInstance()->error("ENopen inp ʧ", "CivHydrCalc", __FUNCTION__);
        return false;
    }
    CivSysLog::getInstance()->info("ENopen inp ɹ", "CivHydrCalc", __FUNCTION__);
    if (ENopenH() > 0)
    {
        ENclose();
        CivSysLog::getInstance()->error("ENopenH inp ʧ", "CivHydrCalc", __FUNCTION__);
        return false;
    }
 
    if (ENinitH(0) > 0) 
    {
        ENclose();
        CivSysLog::getInstance()->error("ENinitH inp ʧ", "CivHydrCalc", __FUNCTION__);
        return false;
    }
    CivSysLog::getInstance()->info("ENinitH inp ɹ", "CivHydrCalc", __FUNCTION__);

    float dHStep;
    ENGetVal(4, &dHStep);

   //  hydrCompute(errcode);
    bool isFirst = true;
    // 
    do
    {
        int nLinkCount;
        ENgetcount(EN_LINKCOUNT, &nLinkCount);
        /*for (int i = 1; i <= nLinkCount; i++)
        {
            int linkType;
            char id[128] = "";
            ENgetlinktype(i, &linkType);
            ENgetlinkid(i, id);
        }*/

        if (ENrunH(&t) > 100)	// errcode > 100 Ǵ
        {
            ENclose();
            return false;
        }
        ENnextH(&tstep);
        if (isFirst)
        {
            mDbConn->deleteByDate(NODERESULTTABLE, "", CurrentDate);
            mDbConn->deleteByDate(PIPERESULTTABLE, "", CurrentDate);
            isFirst = false;
        }
        // ȡǰڵ͹ܶ
        getNodeResult(iTime);
        getLinkResult(iTime);
        iTime++;   
    } while (tstep > 0);
    // 
    if (!mResultCache.updateToDb(mDbConn))
    {
        CivSysLog::getInstance()->error("ˮ洢ʧ", "CivHydrCalc", __FUNCTION__);
        return false;
    }
    // ¹ݿ
    (void)mDbConn->updateNodeByInterval(CurrentDate, "0");
    (void)mDbConn->updateLinkByInterval(CurrentDate,"0");

    ENcloseH();
    ENclose();
    return true;
}

bool CivHydrCalc::qualitySimulation(char* inpFile, char* rptFile, char* binOutFile)
{
    CivSysLog::getInstance()->error("ʼˮʷ", "CivHydrCalc", __FUNCTION__);
    if (!exportInp(inpFile))
    {
        CivSysLog::getInstance()->error("дinpļʧ", "CivHydrCalc", __FUNCTION__);
        return false;
    }
    CivSysLog::getInstance()->info("дinpɹ", "CivHydrCalc", __FUNCTION__);


    // ˮҪȽˮ,ûֱӲENSolveH Ϊ˿Ǽѹѹ
    long  t(0), tstep(0);
    int   iTime(0);
 
    int  errcode = 0;


    errcode = ENopen(inpFile, rptFile, binOutFile);
    if (errcode > 0)
    {
        ENclose();
        CivSysLog::getInstance()->error("ENopen inp ʧ", "CivHydrCalc", __FUNCTION__);
        return false;
    }
    CivSysLog::getInstance()->info("ENopen inp ɹ", "CivHydrCalc", __FUNCTION__);
    if (ENopenH() > 0)
    {
        ENclose();
        CivSysLog::getInstance()->error("ENopenH inp ʧ", "CivHydrCalc", __FUNCTION__);
        return false;
    }

    if (ENinitH(EN_SAVE) > 0)
    {
        ENclose();
        CivSysLog::getInstance()->error("ENinitH inp ʧ", "CivHydrCalc", __FUNCTION__);
        return false;
    }
    CivSysLog::getInstance()->info("ENinitH inp ɹ", "CivHydrCalc", __FUNCTION__);
    bool isFirst = true;
    do
    {
        int nLinkCount;
        ENgetcount(EN_LINKCOUNT, &nLinkCount);
     
        errcode = ENrunH(&t);	// errcode > 100 Ǵ
        if (errcode > 100)
        {
            ENopenQ();
            ENclose();
            return false;
        }
        if (isFirst)
        {
            mDbConn->deleteByDate(NODERESULTTABLE, "", CurrentDate);
            mDbConn->deleteByDate(PIPERESULTTABLE, "", CurrentDate);
            isFirst = false;
        }
        // ȡǰڵ͹ܶ
        getNodeResult(iTime);
        getLinkResult(iTime);
        ENnextH(&tstep);

        iTime++;

    } while (tstep > 0);

    ENcloseH();

    // ˮʷ
    ENopenQ();
    ENinitQ(1);

    t = 0;
    tstep = 0;
    iTime = 0;
    do
    {
        errcode = ENrunQ(&t);	// errcode > 100 Ǵ
        if (errcode > 100)
        {
            ENcloseQ();
            ENclose();
            return false;
        }
  
        // ȡˮģ
        getNodeQuality(iTime);
        getLinkQuality(iTime);

        ENstepQ(&tstep);

        iTime++;

    } while (tstep > 0);


    ENcloseQ();
    ENclose();

    // 
    if (!mResultCache.updateToDb(mDbConn))
    {
        CivSysLog::getInstance()->error("ˮʷ洢ʧ", "CivHydrCalc", __FUNCTION__);
        return false;
    }
    CivSysLog::getInstance()->info("ˮʷ洢ɹ", "CivHydrCalc", __FUNCTION__);

    // ¹ݿ
    (void)mDbConn->updateNodeByInterval(CurrentDate, "0");
    (void)mDbConn->updateLinkByInterval(CurrentDate, "0");

    CivSysLog::getInstance()->info("ˮʷ", "CivHydrCalc", __FUNCTION__);
    return true;
}

bool CivHydrCalc::exportInp(char* fileName)
{
    //1 ,ȡ
    if (!mDbConn)
        return false;

    if (!mDbConn->open())
        return false;
    
   ///*bool pattern = mDbConn->createTable(*CivPatternTableTemp());
   //bool  cii = mDbConn->createTable(OPTIONTTABLE, "public", CivTableTemp::PARAMETER_TABLE);
   //bool curve = mDbConn->createTable(CURVETABLE, "public", CivTableTemp::CURVE_TABLE);
   //bool node = mDbConn->createTable(NODERESULTTABLE, "public", CivTableTemp::NODE_TABLE);
   //bool link = mDbConn->createTable(PIPERESULTTABLE, "public", CivTableTemp::PIPE_TABLE);*/

    // ȡ
    Tables tables = mDbConn->getTables(mNetname);
    if(tables.size()<=0)
        CivSysLog::getInstance()->error(mNetname+ "ѯ", "CivHydrCalc", __FUNCDNAME__);

    //2,
    // ˳: ߷
    CivInp civInp(CivInp::HOURS);

    int total = tables.size();
    for (int i=0;i<total;i++)
    {
        std::string table = tables[i];
        std::cout << table << std::endl;
        if (PIPELINE == table)
            continue;

        assemble(table,civInp);
    }
    assemble(PIPELINE, civInp);

 
    // 
    // Componets curves = mDbConn->getComponets(CURVETABLE);

    // ʼ
    Options options;
    options.units = Options::UNITS::LPS;
    options.TRIALS = "40";
    options.ACCURACY = "0.001 ";
    options.EMITTEREXPONENT = "0.5";
    options.SpecificGravity = "0.998";
    options.headLoss = Options::HEADLOSS::HW;
    options.TOLERANCE = "0.0166666666666667 ";
    options.quality = Options::QUALITY::AGE;

     std::map<std::string, std::string> optMap;
    optMap.insert(std::pair<std::string, std::string>("DAMPLIMIT","0"));
    optMap.insert(std::pair<std::string, std::string>("CHECKFREQ", "2"));
    optMap.insert(std::pair<std::string, std::string>("MAXCHECK", "10"));

    civInp.setOption(options, optMap);

    // 
    civInp.setDuration(24);
    civInp.setTimestep("PATTERN", 1);
    civInp.setTimestep("HYDRAULIC", 1);
    civInp.setTimestep("QUALITY", 1);

    civInp.setTimeStart("REPORT", 1);

    civInp.writeToFile(fileName);
    return false;
}

void CivHydrCalc::assemble(const std::string& table, CivInp& inp)
{    
    // 齨ͽֶ
    if (PIPELINE == table)
    {
        // 
        //;ID  Node1   Node2   Length   Diameter  	Roughness  MinorLoss  Status
        // ر
        Componets comps = mDbConn->getComponets(table);
        PipeFields pipes;
        Civitems items;
        int total = comps.size();
        for (int i =0 ;i< total;i++)
        {
            Componet comp = comps[i];

            // ˵뷧źˮӵĹ
            Str number = comp.find("id")->second;
            FiledFilter filter = mDbConn->getFilter();
            auto iter = filter.find(PIPELINE);
            if (iter != filter.end())
            {
                auto lineFilter = iter->second;
                if (lineFilter.count(number) > 0)
                    continue;
            }
           
            Str id = comp.find(pipes.ID)->second;
            Str startNode = comp.find(pipes.Node1)->second;
            Str endNode = comp.find(pipes.Node2)->second;
            Str length = comp.find(pipes.Length)->second;
            Str diameter = comp.find(pipes.Diameter)->second;
            Str roughness = comp.find(pipes.Roughness)->second;
            Str minorLoss = comp.find(pipes.MinorLoss)->second;
            Str status = comp.find(pipes.Status)->second;

            // ע˳
            CivItem civItem;
            if(id==""||id.empty())
                id = "GD" + comp.find("id")->second;

            civItem.push_back(id);
            civItem.push_back(startNode);
            civItem.push_back(endNode);
            civItem.push_back(length);
            civItem.push_back(diameter);
            civItem.push_back(roughness);
            civItem.push_back(minorLoss);
            civItem.push_back(status);
            items.push_back(civItem);
        }
        inp.writeComponet(items,_PIPES);
       
    }
    else if (PIPENODE == table)
    {
        Componets comps = mDbConn->getComponets(table);
        // ڵ
        // ;ID              	Elev        	Demand      	Pattern
         //   
        //;Node            	X-Coord         	Y-Coord
        JunctionsFields jontionFields;
        Civitems jounItems;
        Civitems nodeCoordItems;
        Civitems patternItems;
        Civitems initQaulityItems;

        int total = comps.size();
        std::set<Str> patterNams;
        for (int i = 0; i < total; i++)
        {
            Componet comp = comps[i];
            Str id = comp.find(jontionFields.ID)->second;
            Str elevation = comp.find(jontionFields.Elev)->second;
            Str demand = comp.find(jontionFields.Demand)->second;
            Str pattern = comp.find(jontionFields.Pattern)->second;
            Str x_coord = comp.find(jontionFields.X_Coord)->second;
            Str y_coord = comp.find(jontionFields.Y_Coord)->second;

            // ע˳
            //1, ڵ
            CivItem civItem;
            civItem.push_back(id);
            civItem.push_back(elevation);
            civItem.push_back(demand);
            civItem.push_back(pattern);
      
            jounItems.push_back(civItem);

            // 2, ڵ
            CivItem nodeCoordItem;
            nodeCoordItem.push_back(id);
            nodeCoordItem.push_back(x_coord);
            nodeCoordItem.push_back(y_coord);

            nodeCoordItems.push_back(nodeCoordItem);
            if (!pattern.empty())
            {
                patterNams.insert(pattern);
            }

            // ʼˮ
            Str initQual = comp.find(jontionFields.InitQual)->second;
            CivItem initQualItem;
            initQualItem.push_back(id);
            initQualItem.push_back(initQual);

            initQaulityItems.push_back(initQualItem);           
        }

        Componets patterns = mDbConn->getComponets(PATTERNTABLE);
        PatternFields fileds;
        size_t patternSize = patterns.size();
        for (int i = 0; i < patternSize; i++)
        {
            Componet pattern = patterns[i];
            Str paternName = pattern.find(fileds.Name)->second;
            Str paternValue = pattern.find(fileds.Value)->second;

            CivItem item;
            item.push_back(paternName);
            item.push_back(paternValue);

            patternItems.push_back(item);
        }
        inp.writeComponet(jounItems,_JUNCTIONS);
        inp.writeComponet(nodeCoordItems, _COORDS);
        inp.writeComponet(patternItems, _PATTERNS);
        inp.writeComponet(initQaulityItems, _QUALITY);
    }
    else if (RESIVOIR == table) // ˮԴ
    {
        Componets comps = mDbConn->getComponets(table);
        //;ID              	Head        	Pattern
        Civitems coordItems;
        Civitems resirvoirItems;

        ResivoirFields fields;
        int total = comps.size();
        for (int i = 0; i < total; i++)
        {
            Componet comp = comps[i];
            Str id = comp.find(fields.ID)->second;
            Str elevation = comp.find(fields.Elev)->second;
            Str flow = comp.find(fields.Flow)->second;
            Str head = comp.find(fields.Head)->second;
            Str pattern = comp.find(fields.Pattern)->second;
            Str x_coord = comp.find(fields.X_Coord)->second;
            Str y_coord = comp.find(fields.Y_Coord)->second;

            // ע˳
            //1, coord
            CivItem civItem;
            civItem.push_back(id);
            civItem.push_back(head);
            civItem.push_back(pattern);

            resirvoirItems.push_back(civItem);

            // 2, ڵ
            CivItem nodeCoordItem;
            nodeCoordItem.push_back(id);
            nodeCoordItem.push_back(x_coord);
            nodeCoordItem.push_back(y_coord);

            coordItems.push_back(nodeCoordItem);
        }

        inp.writeComponet(coordItems, _COORDS);
        inp.writeComponet(resirvoirItems, _RESERVOIRS);

    }
    else if (PUMP == table) // ˮ
    {
        //;ID              	Node1           	Node2           	Parameters
        Componets comps = mDbConn->getLikelyPipes(table);
        PumpFields fields;

        Civitems pumpItems;
        int total = comps.size();
        for (int i = 0; i < total; i++)
        {
            Componet comp = comps[i];
            Str id = comp.find(fields.ID)->second;
            Str node1 = comp.find(fields.node1)->second;
            Str node2 = comp.find(fields.node2)->second;

            std::vector<Str> vec;
            Str head = comp.find(fields.head)->second;
            if (!head.empty())
                vec.push_back("HEAD " + head);

            Str power = comp.find(fields.power)->second;
            if (!power.empty())
                vec.push_back("POWER " + power);

            Str pattern = comp.find(fields.pattern)->second;
            if (!pattern.empty())
                vec.push_back("PATTERN " + pattern);

            Str speed = comp.find(fields.speed)->second;
            if (!speed.empty())
                vec.push_back("SPEED " + speed);

            // ˮ
            CivItem tankNode;
            tankNode.push_back(id);
            tankNode.push_back(node1);
            tankNode.push_back(node2);

            // 
            char param[128] = " ";
            int total = vec.size();
            for(int i=0;i<total;i++)
            {
                strcat_s(param, vec[i].c_str());
            }
            tankNode.push_back(param);
            pumpItems.push_back(tankNode);
        }
        inp.writeComponet(pumpItems, _PUMPS);
    }
    else if (TANK == table) // ˮ
    {
        Componets comps = mDbConn->getComponets(table);
        // ; ID              	Elevation   	InitLevel   	MinLevel    	MaxLevel    	Diameter    	MinVol      	VolCurve
        TanksFields tkfFileds;
        Civitems nodeCoordItems;
        Civitems tankItems;
        int total = comps.size();
        for (int i = 0; i < total; i++)
        {
            Componet comp = comps[i];
            Str id = comp.find(tkfFileds.ID)->second;
            Str elevation = comp.find(tkfFileds.Elev)->second;
            Str initLevel = comp.find(tkfFileds.InitLevel)->second;
            Str minLevel = comp.find(tkfFileds.MinLevel)->second;
            Str maxLevel = comp.find(tkfFileds.MaxLevel)->second;
            Str diametor = comp.find(tkfFileds.Diameter)->second;

            Str minVol = comp.find(tkfFileds.MinVol)->second;
       
            Str x_coord = comp.find(tkfFileds.X_Coord)->second;
            Str y_coord = comp.find(tkfFileds.Y_Coord)->second;
          
            // ˮ
            CivItem tankNode;
            tankNode.push_back(id);
            tankNode.push_back(elevation);
            tankNode.push_back(initLevel);
            tankNode.push_back(minLevel);
            tankNode.push_back(maxLevel);
            tankNode.push_back(diametor);
            tankNode.push_back(minVol);
       
            tankItems.push_back(tankNode);

            // 2, ڵ
            CivItem nodeCoordItem;
            nodeCoordItem.push_back(id);
            nodeCoordItem.push_back(x_coord);
            nodeCoordItem.push_back(y_coord);

            nodeCoordItems.push_back(nodeCoordItem);   
        }
        inp.writeComponet(nodeCoordItems, _COORDS);
        inp.writeComponet(tankItems, _TANKS);
    }
    else
    {   
          // 
          //;ID              	Node1           	Node2           	Diameter    	Type	Setting     	MinorLoss  
          Componets comps = mDbConn->getLikelyPipes(table);
          ValvesFields fields;

          Civitems vavleItems;
          int total = comps.size();
          for (int i = 0; i < total; i++)
          {
              Componet comp = comps[i];
              Str id = comp.find(fields.ID)->second;
              Str node1 = comp.find(fields.node1)->second;
              Str node2 = comp.find(fields.node2)->second;
              Str diametor = comp.find(fields.Diameter)->second;
              Str label = comp.find(fields.type)->second;
              Str typeSeting = comp.find(fields.setting)->second;
              Str minorLoss = comp.find(fields.minorLoss)->second;

              // 
              CivItem vavleNode;
              vavleNode.push_back(id);
              vavleNode.push_back(node1);
              vavleNode.push_back(node2);
              vavleNode.push_back(diametor);
              vavleNode.push_back(label);
              vavleNode.push_back(typeSeting);
              vavleNode.push_back(minorLoss);

              vavleItems.push_back(vavleNode);
          }
          inp.writeComponet(vavleItems, _VALVES);
     }
}

bool CivHydrCalc::getDataByInterval(char* date, char* interval)
{
    if (!mDbConn->open())
        return false;

    if (!mDbConn->updateNodeByInterval(date, interval))
        return false;

    if (!mDbConn->updateLinkByInterval(date, interval))
        return false;

    return true;
}

void CivHydrCalc::getNodeResult(short time)
{
    int nNodeCount;
    ENgetcount(EN_NODECOUNT, &nNodeCount);
    
    for (int i = 1; i <= nNodeCount; i++)
    {
        NodeResultItem nodeItem;
        // 
        ENgetnodeid(i, nodeItem.szNo);
        // 
        ENgetnodevalue(i, EN_DEMAND, &nodeItem.dFlow);
        // ˮͷ
        ENgetnodevalue(i, EN_HEAD, &nodeItem.dHead);
        // ѹ
        ENgetnodevalue(i, EN_PRESSURE, &nodeItem.dPressure);
        // 
        ENgetnodevalue(i, EN_ELEVATION, &nodeItem.dElevation);
        // ʵˮ
        ENgetnodevalue(i, EN_DEMAND, &nodeItem.dDemand);
        // ˮ
        ENgetnodevalue(i, EN_TANKVOLUME, &nodeItem.dTankVolume);
        // ˮ
        ENgetnodevalue(i, EN_MAXVOLUME, &nodeItem.dTankMaxVolume);
        // Һλ߶
        ENgetnodevalue(i, EN_TANKLEVEL, &nodeItem.dTankLevel);
      
        nodeItem.dInterval = time;

         // ŵ 
        mResultCache.addNodeItems(nodeItem);
    }
}

void CivHydrCalc::getLinkResult(short time)
{
    int  nLinkCount;
    ENgetcount(EN_LINKCOUNT, &nLinkCount);

    for (int i = 1; i <= nLinkCount; i++)
    {
        LinkResultItem linkItem;

        int *typeCode = (int*)malloc(sizeof(int));
     
        ENgetlinktype(i,typeCode );
        if (*typeCode != EN_PIPE)
        {
            free(typeCode);
            continue;
        }
        free(typeCode);

        // 
        ENgetlinkid(i, linkItem.szNo);
        //  
        ENgetlinkvalue(i, EN_FLOW, &linkItem.dFlow);
        // 
        linkItem.nFlowDirect = (linkItem.dFlow >= 0) ? 0 : 1;
        // 
        ENgetlinkvalue(i, EN_VELOCITY, &linkItem.dVelocity);
        // ˮͷʧ
        ENgetlinkvalue(i, EN_HEADLOSS, &linkItem.dHeadloss);
        // λˮͷʧ
        float dPipeLen = 0.0f;;
        ENgetlinkvalue(i, EN_LENGTH, &dPipeLen);
        linkItem.dUnitHeadloss = (linkItem.dHeadloss / dPipeLen) * 1000;
        // ȡϵͱ
        int node1 = 0;
        int node2 = 0;
        ENgetlinknodes(i, &node1, &node2);
        // ϵˮͷ
        ENgetnodevalue(node1, EN_HEAD, &linkItem.dFromNodHeadloss);
        // ˮͷ
        ENgetnodevalue(node2, EN_HEAD, &linkItem.dToNodHeadloss);
     
        // ֲˮͷʧ
        float fCoeff;
        ENgetlinkvalue(i, EN_MINORLOSS, &fCoeff);
        double dMinorloss = fCoeff * linkItem.dHeadloss;
        linkItem.dLocalHeadloss = dMinorloss;// ֲˮͷʧ 

        // Ħˮͷʧ
        linkItem.dFrictionHeadloss = linkItem.dHeadloss - linkItem.dLocalHeadloss;// ֲˮͷʧ 

        // ״̬
        float fState = 0;
        ENgetlinkvalue(i, EN_STATUS, &fState);
        if ((int)fState == 0)
            strcpy_s(linkItem.szStatus, "ر");
        else
            strcpy_s(linkItem.szStatus, "");

        linkItem.dInterval = time;
        // ŵ 
        mResultCache.addLinkItems(linkItem);
    }
}

void CivHydrCalc::getNodeQuality(short time)
{
    int nNodeCount;
    ENgetcount(EN_NODECOUNT, &nNodeCount);

  
    for (int i = 1; i <= nNodeCount; i++)
    {
        char No[35];
        // 
        ENgetnodeid(i, No);
        float* quality = (float*)malloc(sizeof(float));
        // ˮ
        ENgetnodevalue(i, EN_QUALITY, quality);
    
        mResultCache.addNodeQuality(*quality, time, No);
        free(quality);
    }
}

void CivHydrCalc::getLinkQuality(short time)
{
    int  nLinkCount;
    ENgetcount(EN_LINKCOUNT, &nLinkCount);
    int* typeCode = (int*)malloc(sizeof(int));
    float* quality = (float*)malloc(sizeof(float));
    for (int i = 1; i <= nLinkCount; i++)
    {
      
        ENgetlinktype(i, typeCode);
        if (*typeCode != EN_PIPE)
        {
            continue;
        }
        char lnkNo[35];
        // 
        ENgetlinkid(i, lnkNo);
        
        // ˮ
        ENgetlinkvalue(i, EN_LINKQUAL, quality);  
        mResultCache.addLinkQuality(*quality, time, lnkNo);       
    } 
    free(typeCode);
    free(quality);
}