#include "CivHydrSimulation.h"
#include "CivPgConn.h"
#include "CivNewInp.h"
#include "CivBuilder.h"
#include "CivInpDirector.h"
#include "CivSysLog.h"
#include "epanet2.h"
#include "../pandaDbManager/CivCommonUtils.h"
#include "CivTrackingResultCache.h"


CivHydrSimulation::CivHydrSimulation()
   :mDbConn(nullptr),
	mInpFile("test.inp"),
	mRptFile("test.rpt"),
	mBinFile("test.bin")
{

}

CivHydrSimulation::CivHydrSimulation(CivDbConn* dbConn)
	:mDbConn(dbConn),
	mInpFile("test.inp"),
	mRptFile("test.rpt"), 
	mBinFile("test.bin")
{
    if (dbConn)
        dbConn->checkHyDrNetTable();
}

CivHydrSimulation::~CivHydrSimulation()
{

}

char* CivHydrSimulation::getInpFile()
{ 
	return const_cast<char*>(mInpFile.c_str()); 
}

void CivHydrSimulation::setInpFile(const std::string& inpFile)
{ 
	mInpFile = inpFile; 
}

char* CivHydrSimulation::getRptFile()
{ 
	return const_cast<char*>(mRptFile.c_str()); 
}

void CivHydrSimulation::setRptFile(const std::string& rptFile)
{ 
	mRptFile = rptFile; 
}

char* CivHydrSimulation::getBinFile()
{
	return const_cast<char*>(mBinFile.c_str());
}

std::string CivHydrSimulation::getAnalysisResults()
{
    std::string result;
    std::ifstream myfile(getRptFile());

    if (myfile.is_open())
    {
        return result;
    }

    while (!myfile.eof())
    {
        myfile >> result;
    }

    myfile.close();

    return result;
}

void CivHydrSimulation::setBinFile(const std::string& binFile)
{ 
	mBinFile = binFile; 
}

void CivHydrSimulation::registDb(CivDbConn* dbConn)
{
	mDbConn = dbConn;
    if (mDbConn)
        mDbConn->checkHyDrNetTable();
}

bool CivHydrSimulation::convertInp(const QualityAnalyType analyType, const std::string& qulityName)
{
	CivBuilder* builder = new CivInpBuilder();
    builder->setQualityType(analyType, qulityName);
	builder->registDb(mDbConn);

	CivInpDirector director;
	director.setBuilder(builder);
	director.create();

	CivNewInp* newInp = builder->getNewInp();
	char* inpFileName = getInpFile();

	if (inpFileName 
		&& (*inpFileName !='\0') 
		&& !newInp->writeToFile(inpFileName))
	{
		delete builder;
		return false;
	}

	delete builder;
	return true;
}

bool CivHydrSimulation::hdyrSimulation()
{
	CivSysLog::getInstance()->error("ʼˮ", "CivHydrSimulation", __FUNCTION__);
	if (!convertInp(ANALYSIS_RESERVE,""))
	{
		CivSysLog::getInstance()->error("дinpļʧ", "CivHydrSimulation", __FUNCTION__);
		return false;
	}
	CivSysLog::getInstance()->info("дinpɹ", "CivHydrSimulation", __FUNCTION__);

    short hydCode = 1;
    long  t(0), tstep(0);
    int   iTime(0);

    int  errcode = 0;

    char* fileName = getInpFile();
    char* rptFile = getRptFile();
    char* binOutFile = getBinFile();

    errcode = ENopen(fileName, rptFile, binOutFile);
    if (errcode > 0)
    {
        ENclose();
        CivSysLog::getInstance()->error("ENopen inp ʧ", "CivHydrCalc", __FUNCTION__);
        return false;
    }
    CivSysLog::getInstance()->info("ENopen inp ɹ", "CivHydrCalc", __FUNCTION__);
    if (ENopenH() > 0)
    {
        ENclose();
        CivSysLog::getInstance()->error("ENopenH inp ʧ", "CivHydrCalc", __FUNCTION__);
        return false;
    }

    if (ENinitH(0) > 0)
    {
        ENclose();
        CivSysLog::getInstance()->error("ENinitH inp ʧ", "CivHydrCalc", __FUNCTION__);
        return false;
    }
    CivSysLog::getInstance()->info("ENinitH inp ɹ", "CivHydrCalc", __FUNCTION__);

    float dHStep;
    ENGetVal(4, &dHStep);

    // 
    do
    {
        int nLinkCount;
        ENgetcount(EN_LINKCOUNT, &nLinkCount);
        /*for (int i = 1; i <= nLinkCount; i++)
        {
            int linkType;
            char id[128] = "";
            ENgetlinktype(i, &linkType);
            ENgetlinkid(i, id);
        }*/

        if (ENrunH(&t) > 100)	// errcode > 100 Ǵ
        {
            ENclose();
            return false;
        }

        ENnextH(&tstep);
      
        // ȡǰڵ͹ܶ
        getNodeResult(iTime);
        getLinkResult(iTime);
        iTime++;
    } while (tstep > 0);

    ENcloseH();
    ENclose();
    //ģˢ
    if (!brushingResult())
    {
        CivSysLog::getInstance()->error("ˮ洢ʧ", "CivHydrCalc", __FUNCTION__);
        return false;
    }
      
    return true;
}

bool CivHydrSimulation::brushingResult()
{
    clearSamePeriodValue(mDbConn, CurrentDate);

    // 
    if (!mResultCache.updateToDb(mDbConn))
    {
        CivSysLog::getInstance()->error("ˮ洢ʧ", "CivHydrCalc", __FUNCTION__);
        return false;
    }

    // ¹ݿ
    updateLastestDataToGisNet(mDbConn, "0");


    return true;
}

bool  CivHydrSimulation::clearSamePeriodValue(CivDbConn* dbConn, const std::string& date)
{
    if (dbConn == nullptr)
        return false;

    dbConn->deleteByField(NODERESULTTABLE, "", date);
    dbConn->deleteByField(PIPERESULTTABLE, "", date);

    return true;
}

bool CivHydrSimulation::updateLastestDataToGisNet(CivDbConn* dbConn, const std::string& time)
{
    if (dbConn == nullptr)
        return false;

    dbConn->updateNodeByInterval(CurrentDate, time);
    dbConn->updateLinkByInterval(CurrentDate, time);

    return true;
}

bool CivHydrSimulation::qualitySimulation()
{
    CivSysLog::getInstance()->error("ʼˮʷ", "qualitySimulation", __FUNCTION__);
    if (!convertInp(ANALYSIS_AGE, ""))
    {
        CivSysLog::getInstance()->error("дinpļʧ", "qualitySimulation", __FUNCTION__);
        return false;
    }
    CivSysLog::getInstance()->info("дinpɹ", "qualitySimulation", __FUNCTION__);

    // ˮʼ
    if (!qualityCompute())
    {
        CivSysLog::getInstance()->error("ˮʷʧ", "CivHydrCalc", __FUNCTION__);
        return false;
    }

    //ģˢ
    if (!brushingResult())
    {
        CivSysLog::getInstance()->error("ˮʷ洢ʧ", "CivHydrCalc", __FUNCTION__);
        return false;
    }
    return true;
}

bool  CivHydrSimulation::trackingSimulation(char* snNode, int hour)
{
    CivSysLog::getInstance()->error("ʼˮʷ", "qualitySimulation", __FUNCTION__);
    setQualityAnalyType(ANALYSIS_TRACE);
    if (!convertInp(ANALYSIS_TRACE, snNode))
    {
        CivSysLog::getInstance()->error("дinpļʧ", "qualitySimulation", __FUNCTION__);
        return false;
    }
    CivSysLog::getInstance()->info("дinpɹ", "qualitySimulation", __FUNCTION__);

    // ˮʼ
    if (!qualityCompute())
    {
        CivSysLog::getInstance()->error("׷ٷʧ", "CivHydrCalc", __FUNCTION__);
        return false;
    }
      
    // ɢ洢
    //ģˢ
    if (!brushingResult())
    {
        CivSysLog::getInstance()->error("׷ٷʧ", "CivHydrCalc", __FUNCTION__);
        return false;
    }


    return true;
}

bool CivHydrSimulation::qualityCompute()
{
    // ˮҪȽˮ,ûֱӲENSolveH Ϊ˿Ǽѹѹ
    long  t(0), tstep(0);
    int   iTime(0);

    int  errcode = 0;

    char* fileName = getInpFile();
    char* rptFile = getRptFile();
    char* binOutFile = getBinFile();

    errcode = ENopen(fileName, rptFile, binOutFile);
    if (errcode > 0)
    {
        ENclose();
        CivSysLog::getInstance()->error("ENopen inp ʧ", "CivHydrCalc", __FUNCTION__);
        return false;
    }
    CivSysLog::getInstance()->info("ENopen inp ɹ", "CivHydrCalc", __FUNCTION__);
    if (ENopenH() > 0)
    {
        ENclose();
        CivSysLog::getInstance()->error("ENopenH inp ʧ", "CivHydrCalc", __FUNCTION__);
        return false;
    }

    if (ENinitH(EN_SAVE) > 0)
    {
        ENclose();
        CivSysLog::getInstance()->error("ENinitH inp ʧ", "CivHydrCalc", __FUNCTION__);
        return false;
    }
    CivSysLog::getInstance()->info("ENinitH inp ɹ", "CivHydrCalc", __FUNCTION__);

    do
    {
        int nLinkCount;
        ENgetcount(EN_LINKCOUNT, &nLinkCount);
        /*for (int i = 1; i <= nLinkCount; i++)
        {
            int linkType;
            char id[128] = "";
            ENgetlinktype(i, &linkType);
            ENgetlinkid(i, id);
        }*/

        errcode = ENrunH(&t);	// errcode > 100 Ǵ
        if (errcode > 100)
        {
            ENcloseH();
            ENclose();
            return false;
        }
   
        // ȡǰڵ͹ܶ
        getNodeResult(iTime);
        getLinkResult(iTime);

        ENnextH(&tstep);

        iTime++;

    } while (tstep > 0);

    ENcloseH();

    // ˮʷ
    ENopenQ();
    ENinitQ(1);

    t = 0;
    tstep = 0;
    iTime = 0;
    do
    {
        errcode = ENrunQ(&t);	// errcode > 100 Ǵ
        if (errcode > 100)
        {
            ENcloseQ();
            ENclose();
            return false;
        }

        // ȡˮģ
        getNodeQuality(iTime);
        getLinkQuality(iTime);

        ENstepQ(&tstep);

        iTime++;

    } while (tstep > 0);


    ENcloseQ();
    ENclose();

    CivSysLog::getInstance()->info("ˮʷ", "CivHydrCalc", __FUNCTION__);
    return true;
}

void  CivHydrSimulation::getTrackingResult(int hours, std::string& newresult)
{

    std::string result;
    result.append("[");

    // ڵ͹ܶοռϢ
    CivTrackingResultCache trackingCache;
    trackingCache.cacheInfoFromDb(mDbConn);

    std::stringstream stream;
    for (int i = 0; i < hours; i++)
    {
        NodeResultItems nodeItemsMap;
        LinkResultItems linkItemsMap;
        mResultCache.getResultByTime(i, nodeItemsMap, linkItemsMap);

        // ҽڵ׷ٵĽڵ
        std::string nodeTempText;
        nodeTempText.append("{\"type\":\"ȾԴ\",\"location\":[");

        auto nodeIter = nodeItemsMap.begin();
        size_t nodeTotal = nodeItemsMap.size();
        while (nodeIter != nodeItemsMap.end())
        {

            if (nodeIter->second.dQuality > 0)
            {
                nodeTempText.append("{\"value\":[");
                std::string sn = nodeIter->first;

                Junctions junction;
                trackingCache.getNodeInfoBySn(sn, junction);
             
                nodeTempText.append(junction.xCoord);
                nodeTempText.append(",");
                nodeTempText.append(junction.yCoord);
                nodeTempText.append("],");

                nodeTempText.append("\"code\":");
                nodeTempText.append(sn.c_str());
                nodeTempText.append("},");
            }

            nodeIter++;
        }
        std::string newNodeTempText = nodeTempText.substr(0, nodeTempText.length() - 1);
        newNodeTempText.append("]}");

        // ׷ٹܶα
        auto linkIter = linkItemsMap.begin();
        size_t total = linkItemsMap.size();
        int index = 1;

        std::string linkTempText;
        linkTempText.append("{\"type\":\"Ⱦܶ\",\"location\":[");

        while (linkIter != linkItemsMap.end())
        {
            if (linkIter->second.dQuality > 0)
            {
                std::string sn = linkIter->first;
                linkTempText.append("{\"coords\":[");


                Pipes pipe;
                trackingCache.getPipeInfoBySn(sn, pipe);

                linkTempText.append("[");
                linkTempText.append(pipe.startX);
                linkTempText.append(",");
                linkTempText.append(pipe.startY);
                linkTempText.append("],");

                linkTempText.append("[");
                linkTempText.append(pipe.endX);
                linkTempText.append(",");
                linkTempText.append(pipe.endY);
                linkTempText.append("]");

                linkTempText.append("]},");
            }
            linkIter++;
        }
        std::string newLinkTempText = linkTempText.substr(0, linkTempText.length() - 1);
        newLinkTempText.append("]}");
  
       result.append("[");
       result.append(newNodeTempText);
       result.append(",");
       result.append(newLinkTempText);
       result.append("],");
    }
    newresult = result.substr(0, result.length() -1 );
    newresult.append("]");
}

void CivHydrSimulation::getNodeResult(short time)
{
    int nNodeCount;
    ENgetcount(EN_NODECOUNT, &nNodeCount);

    for (int i = 1; i <= nNodeCount; i++)
    {
        NodeResultItem nodeItem;
        // 
        ENgetnodeid(i, nodeItem.szNo);
        // 
        ENgetnodevalue(i, EN_DEMAND, &nodeItem.dFlow);
        // ˮͷ
        ENgetnodevalue(i, EN_HEAD, &nodeItem.dHead);
        // ѹ
        ENgetnodevalue(i, EN_PRESSURE, &nodeItem.dPressure);
        // 
        ENgetnodevalue(i, EN_ELEVATION, &nodeItem.dElevation);
        // ʵˮ
        ENgetnodevalue(i, EN_DEMAND, &nodeItem.dDemand);
        // ˮ
        ENgetnodevalue(i, EN_TANKVOLUME, &nodeItem.dTankVolume);
        // ˮ
        ENgetnodevalue(i, EN_MAXVOLUME, &nodeItem.dTankMaxVolume);
        // Һλ߶
        ENgetnodevalue(i, EN_TANKLEVEL, &nodeItem.dTankLevel);

        nodeItem.dInterval = time;

        // ŵ 
        mResultCache.addNodeItems(nodeItem);
    }
}

void CivHydrSimulation::getLinkResult(short time)
{
    int  nLinkCount;
    ENgetcount(EN_LINKCOUNT, &nLinkCount);

    for (int i = 1; i <= nLinkCount; i++)
    {
        LinkResultItem linkItem;

        int* typeCode = (int*)malloc(sizeof(int));

        ENgetlinktype(i, typeCode);
        if (*typeCode != EN_PIPE)
        {
            free(typeCode);
            continue;
        }
        free(typeCode);

        // 
        ENgetlinkid(i, linkItem.szNo);
        //  
        ENgetlinkvalue(i, EN_FLOW, &linkItem.dFlow);
        // 
        linkItem.nFlowDirect = (linkItem.dFlow >= 0) ? 0 : 1;
        // 
        ENgetlinkvalue(i, EN_VELOCITY, &linkItem.dVelocity);
        // ˮͷʧ
        ENgetlinkvalue(i, EN_HEADLOSS, &linkItem.dHeadloss);
        // λˮͷʧ
        float dPipeLen = 0.0f;;
        ENgetlinkvalue(i, EN_LENGTH, &dPipeLen);
        linkItem.dUnitHeadloss = (linkItem.dHeadloss / dPipeLen) * 1000;
        // ȡϵͱ
        int node1 = 0;
        int node2 = 0;
        ENgetlinknodes(i, &node1, &node2);
        // ϵˮͷ
        ENgetnodevalue(node1, EN_HEAD, &linkItem.dFromNodHeadloss);
        // ˮͷ
        ENgetnodevalue(node2, EN_HEAD, &linkItem.dToNodHeadloss);

        // ֲˮͷʧ
        float fCoeff;
        ENgetlinkvalue(i, EN_MINORLOSS, &fCoeff);
        double dMinorloss = fCoeff * linkItem.dHeadloss;
        linkItem.dLocalHeadloss = dMinorloss;// ֲˮͷʧ 

        // Ħˮͷʧ
        linkItem.dFrictionHeadloss = linkItem.dHeadloss - linkItem.dLocalHeadloss;// ֲˮͷʧ 

        // ״̬
        float fState = 0;
        ENgetlinkvalue(i, EN_STATUS, &fState);
        if ((int)fState == 0)
            strcpy_s(linkItem.szStatus, "ر");
        else
            strcpy_s(linkItem.szStatus, "");

        linkItem.dInterval = time;
        // ŵ 
        mResultCache.addLinkItems(linkItem);
    }
}

void CivHydrSimulation::getNodeQuality(short time)
{
    int nNodeCount;
    ENgetcount(EN_NODECOUNT, &nNodeCount);


    for (int i = 1; i <= nNodeCount; i++)
    {
        char No[35];
        // 
        ENgetnodeid(i, No);
        float* quality = (float*)malloc(sizeof(float));

        // ˮ
        ENgetnodevalue(i, EN_QUALITY, quality);

        int trueQuality = *quality;
        if (mQuaLityType == ANALYSIS_TRACE)
            trueQuality *= 100;

        mResultCache.addNodeQuality(trueQuality, time, No);
        free(quality);
    }
}

void CivHydrSimulation::getLinkQuality(short time)
{
    int  nLinkCount;
    ENgetcount(EN_LINKCOUNT, &nLinkCount);
    int* typeCode = (int*)malloc(sizeof(int));
    float* quality = (float*)malloc(sizeof(float));
    for (int i = 1; i <= nLinkCount; i++)
    {

        ENgetlinktype(i, typeCode);
        if (*typeCode != EN_PIPE)
        {
            continue;
        }
        char lnkNo[35];
        // 
        ENgetlinkid(i, lnkNo);

        // ˮ
        ENgetlinkvalue(i, EN_LINKQUAL, quality);

        int trueQuality = *quality;
        if (mQuaLityType == ANALYSIS_TRACE)
            trueQuality *= 100;

        mResultCache.addLinkQuality(trueQuality, time, lnkNo);
    }
    free(typeCode);
    free(quality);
}
